from SentiNet import PolarityType


class SentiSynSet(object):

    __id: str
    __positiveScore: float
    __negativeScore: float

    """
    Constructor of SentiSynSet. Gets input id, positiveScore, negativeScore and sets all corresponding attributes.

    PARAMETERS
    ----------
    id : str
        Id of the SentiSynSet.
    positiveScore : float
        Positive score of the SentiSynSet.
    negativeScore : float
        Negative score of the SentiSynSet.
    """
    def __init__(self, id: str, positiveScore: float, negativeScore: float):
        self.__id = id
        self.__positiveScore = positiveScore
        self.__negativeScore = negativeScore

    """
    Accessor for the positiveScore attribute.
    
    RETURNS
    -------
    float
        PositiveScore of the SentiSynSet.
    """
    def getPositiveScore(self) -> float:
        return self.__positiveScore

    """
    Accessor for the negativeScore attribute.
    
    RETURNS
    -------
    float
        NegativeScore of the SentiSynSet.
    """
    def getNegativeScore(self) -> float:
        return self.__negativeScore

    """
    Accessor for the id attribute.
    
    RETURNS
    -------
    str
        Id of the SentiSynSet.
    """
    def getId(self) -> str:
        return self.__id

    """
    Returns the polarityType of the sentiSynSet. If the positive score is larger than the negative score, the polarity 
    is positive; if the negative score is larger than the positive score, the polarity is negative; if both positive
    score and negative score are equal, the polarity is neutral.
    
    RETURNS
    -------
    PolarityType
        PolarityType of the sentiSynSet.
    """
    def getPolarity(self) -> PolarityType.PolarityType:
        if self.__positiveScore > self.__negativeScore:
            return PolarityType.PolarityType.POSITIVE
        else:
            if self.__positiveScore < self.__negativeScore:
                return PolarityType.PolarityType.NEGATIVE
            else:
                return PolarityType.PolarityType.NEUTRAL

    """
    Method to write SynSets to the specified file in the XML format.

    PARAMETERS
    ----------
    outfile : file 
        File to write XML files
    """
    def saveAsXml(self, outfile):
        outfile.write("<SYNSET>")
        outfile.write("<ID>" + self.__id + "</ID>")
        outfile.write("<PSCORE>" + self.__positiveScore.__str__() + "</PSCORE>")
        outfile.write("<NSCORE>" + self.__negativeScore.__str__() + "</NSCORE>")
        outfile.write("</SYNSET>\n")



from SentiNet import SentiSynSet
from SentiNet import PolarityType
import xml.etree.ElementTree


class SentiNet(object):

    __sentiSynSetList: dict

    """
    Constructor of Turkish SentiNet. Reads the turkish_sentinet.xml file from the resources directory. For each
    sentiSynSet read, it adds it to the sentiSynSetList.
    """
    def __init__(self, fileName=None):
        self.__sentiSynSetList = {}
        if fileName is None:
            fileName = "turkish_sentinet.xml"
        root = xml.etree.ElementTree.parse(fileName).getroot()
        for sentiSynSet in root:
            id = ""
            positiveScore = 0.0
            negativeScore = 0.0
            for part in sentiSynSet:
                if part.tag == "ID":
                    id = part.text
                else:
                    if part.tag == "PSCORE":
                        positiveScore = part.text
                    else:
                        negativeScore = part.text
            if id != "":
                self.__sentiSynSetList[id] = SentiSynSet.SentiSynSet(id, positiveScore, negativeScore)

    """
    Accessor for a single SentiSynSet.
    
    PARAMETERS
    ----------
    id : str
        Id of the searched SentiSynSet.
        
    RETURNS
    -------
    SentiSynSet
        SentiSynSet with the given id.
    """
    def getSentiSynSet(self, id : str) -> SentiSynSet:
        return self.__sentiSynSetList[id]

    """
    Adds specified SentiSynSet to the SentiSynSet list.

    PARAMETERS
    ----------
    sentiSynSet : SentiSynSet
        SentiSynSet to be added
    """
    def addSentiSynSet(self, sentiSynSet : SentiSynSet):
        self.__sentiSynSetList[sentiSynSet.getId()] = sentiSynSet

    """
    Removes specified SentiSynSet from the SentiSynSet list.

    PARAMETERS
    ----------
    sentiSynSet : SentiSynSet
        SentiSynSet to be removed
    """
    def removeSentiSynSet(self, sentiSynSet : SentiSynSet):
        del self.__sentiSynSetList[sentiSynSet.getId()]

    """
    Constructs and returns a list of ids, which are the ids of the SentiSynSets having polarity
    polarityType.
    
    PARAMETERS
    ----------
    polarityType : PolarityType
        PolarityTypes of the searched SentiSynSets
        
    RETURNS
    -------
    list
        A list of id having polarityType polarityType.
    """
    def getPolarity(self, polarityType : PolarityType.PolarityType) -> list:
        result = []
        for sentiSynSet in self.__sentiSynSetList.values():
            if sentiSynSet.getPolarity() == polarityType:
                result.append(sentiSynSet.getId())
        return result

    """
    Returns the ids of all positive SentiSynSets.
    
    RETURNS
    -------
    list
        A list of ids of all positive SentiSynSets.
    """
    def getPositives(self) -> list:
        return self.getPolarity(PolarityType.PolarityType.POSITIVE)

    """
    Returns the ids of all negative SentiSynSets.

    RETURNS
    -------
    list
        A list of ids of all negative SentiSynSets.
    """

    def getNegatives(self) -> list:
        return self.getPolarity(PolarityType.PolarityType.NEGATIVE)

    """
    Returns the ids of all neutral SentiSynSets.

    RETURNS
    -------
    list
        A list of ids of all neutral SentiSynSets.
    """

    def getNeutrals(self) -> list:
        return self.getPolarity(PolarityType.PolarityType.NEUTRAL)

    """
    Method to write SynSets to the specified file in the XML format.

    PARAMETERS
    ----------
    fileName : str
        file name to write XML files
    """
    def saveAsXml(self, fileName: str):
        outfile = open(fileName, 'w', encoding='utf8')
        outfile.write("<SYNSETS>\n")
        for synSet in self.__sentiSynSetList.values():
            synSet.saveAsXml(outfile)
        outfile.write("</SYNSETS>\n")
        outfile.close()


from setuptools import setup

setup(
    name='NlpToolkit-SentiNet',
    version='1.0.6',
    packages=['SentiNet'],
    url='https://github.com/olcaytaner/TurkishSentiNet-Py',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='Turkish SentiNet'
)
